{$N+}
unit  realpoly;

interface

type 
	float = extended;
	polynomial = ^term;
	term = record
		degree: integer;
		coefficient: float;
		next: polynomial
	end;

function degree_poly(p: polynomial): integer;
function leading_coeff(p: polynomial): float;
procedure insert_term(t: term; var p: polynomial; augment: boolean);
procedure load(var p: polynomial);
procedure recycle(var p: polynomial);
function copy_poly(p: polynomial): polynomial;
procedure print_poly(p: polynomial);
function mult_by_scalar(c: float;  p: polynomial): polynomial;
function sum_poly(p, q: polynomial): polynomial;
function product_poly(p, q: polynomial): polynomial;
function derivative_poly(p: polynomial): polynomial;
procedure division_poly(f, g: polynomial; var  q, r: polynomial);
function value_poly(p: polynomial; c: float): float;
procedure evaluate_poly(p: polynomial; c: float; var D0, D1: float);

implementation

function degree_poly(p: polynomial): integer;
begin
	if p = nil then 
		degree_poly := -1
	else 
		degree_poly := p^.degree;
end;

function leading_coeff(p: polynomial): float;
begin
	if p = nil then 
		leading_coeff := 0.0
	else 
		leading_coeff := p^.coefficient;
end;

procedure insert_term(t: term; var p: polynomial; augment: boolean);
var 
	run, temp: polynomial;
begin
	if p = nil then
	begin
		new(p);
		p^ := t;
		p^.next := nil;
	end
	else
	begin
		run := p;
		while (run^.degree > t.degree) and (run^.next <> nil) do
			run := run^.next;
		if run^.degree < t.degree then
		begin
			new(temp);
			temp^ := run^;
			run^ := t;
			run^.next := temp;
		end
		else 
			if run^.degree = t.degree then
			begin
				if augment then
					run^.coefficient := run^.coefficient + t.coefficient
				else 
					run^.coefficient := t.coefficient;
			end
			else
			begin
				new(run^.next);
				run := run^.next;
				run^ := t;
				run^.next := nil;
			end;
	end;
end;

procedure load(var p: polynomial);
var 
	t: term; 
	k: integer;
begin
	p := nil;
	t.next := nil;
	repeat
		write(' ⥯  童 ( -1  室): ');
		readln(k);
		if k >= 0 then
		begin
			with t do
			begin
				degree := k;
				write(' 樥: ');
				readln(coefficient);
			end;
			insert_term(t, p, false);
		end;
	until k < 0;
end;

procedure recycle(var p: polynomial);
begin
	if p <> nil then 
	begin
		recycle(p^.next);
		dispose(p);
	end;
	p := nil;
end;

function copy_poly(p: polynomial): polynomial;
var 
	s: polynomial;
begin
	if p = nil then 
		s := nil
	else
	begin
		new(s);
		s^ := p^;
		s^.next := copy_poly(p^.next);
	end;
	copy_poly := s;
end;

procedure print_poly(p: polynomial);
var 
	run: polynomial;
	k: integer;
begin
	if p = nil then 
		writeln(0:2)
	else
	begin
		with p^ do
		begin
			write(coefficient:5:5);
			if degree > 0 then write(' x^', degree);
			run := next;
		end;
		k := 4;
		while run <> nil do
		begin
			dec(k);
			if k = 0 then
			begin
				k := 5;
				writeln;
				write(' ':15);
			end;
			with run^ do
			begin
				if coefficient < 0.0 then 
					write(' - ', -coefficient:5:5)
				else 
					write(' + ', coefficient:5:5);
				if degree > 0 then 
					write(' x^', degree);
				run := next;
			end
		end;
		writeln;
	end
end;

function mult_by_scalar(c: float; p: polynomial): polynomial;
var 
	r, s: polynomial;
begin
	s := copy_poly(p);  
	r := s;
	while r <> nil do
		with r^ do
		begin
			coefficient := c*coefficient;
			r := next
		end;
	mult_by_scalar := s;
end;

function sum_poly(p, q: polynomial): polynomial;
var 
	p_run, s: polynomial;
	t: term;
begin
	s := copy_poly(p);  
	p_run := q;
	while p_run <> nil do
	begin
		t := p_run^;
		t.next := nil;
		insert_term(t, s, true);
		p_run := p_run^.next;
	end;
	sum_poly := s;
end;

function product_poly(p, q: polynomial): polynomial;
var 
	p_run, q_run, s: polynomial;
	t: term;
begin
	s := nil;
	t.next := nil;
	p_run := p;
	while p_run <> nil do
	begin
		q_run := q;
		while q_run <> nil do
		begin
			t.degree := p_run^.degree + q_run^.degree;
			t.coefficient := 
				p_run^.coefficient*q_run^.coefficient;
			insert_term(t, s, true);
			q_run := q_run^.next;
		end;
		p_run := p_run^.next;
	end;
	product_poly := s;
end;

procedure  division_poly(f, g: polynomial;  
		var  q, r: polynomial);
var 
	run, h, temp: polynomial;
	t: term;
	gap, d: word;
	a: float;
begin
	q := nil; 
	d := degree_poly(g);
	r := copy_poly(f);
	while  degree_poly(r) >= d  do
	begin
		gap := degree_poly(r) - d;
		a := leading_coeff(r) / leading_coeff(g);
		h := copy_poly(g); run := h;
		repeat
			with run^ do
			begin
				degree := gap + degree;
				coefficient := -a * coefficient;
				run := next;
			end
		until run = nil;
		temp := sum_poly(r, h);
		recycle(r);  
		recycle(h);
		r := nil;
		t.degree := gap;  t.coefficient := a;
		insert_term(t, q, false);
	end;
end;

function derivative_poly(p: polynomial): polynomial;
var 
	run: polynomial;
begin
	if degree_poly(p) = 0 then 
		derivative_poly := nil
	else
	begin
		run := copy_poly(p);
		derivative_poly := run;
		repeat
			with run^ do
			begin
				coefficient := degree * coefficient;
				dec(degree);
				if next = nil then 
					exit;
				if next^.degree = 0 then
				begin 
					dispose(next); 
					next := nil; 
				end;
				run := next;
			end;
		until run = nil;
	end;
end;

function value_poly(p: polynomial; c: float): float;
var 
	i, k: integer;
	s: float;
	run: polynomial;
begin
	if p = nil then 
		s := 0.0 
	else 
		s := p^.coefficient;
	run := p;
	while run <> nil do
	begin
		if run^.next = nil then 
			k := degree_poly(run)
		else 
			k := degree_poly(run) - degree_poly(run^.next);
		for i := k downto 1 do 
			s := s*c;
		run := run^.next;
		if run <> nil then 
			s := s + run^.coefficient;
	end;
	value_poly := s;
end;

procedure evaluate_poly(p: polynomial;  c: float;  
		var  D0, D1: float);
var 
	i, gap: integer;
	run: polynomial;
begin
	D1 := 0;
	if p = nil then
	begin
		D0 := 0.0;
		exit;
	end;
	D0 := p^.coefficient;
	run := p;
	while run <> nil do
	begin
		if run^.next = nil then 
			gap := degree_poly(run)
		else 
			gap := degree_poly(run) - degree_poly(run^.next);
		for i := gap downto 1 do
		begin
			D1 := D1*c + D0;
			D0 := D0*c;
		end;
		run := run^.next;
		if run <> nil then 
			D0 := D0 + run^.coefficient;
	end;
end;

end.



