program lineclipping;

uses crt, dos;

type
  recttype = record
  x_min, y_min, x_max, y_max : integer;
  end;

screenbuffertype = array[0..63999] of byte;
screenbufferptr = ^screenbuffertype;

  screentype = record
    buffer  : screenbufferptr;
    dbuffer : boolean;
    ytable  : array[0..199] of word;
    width   : word;
    clip    : recttype;
  end;

var
  x1, y1, x2, y2, tmp : integer;
  screen : screentype;
  screen_y : array[0..199] of word;
  i, j : word;

procedure calcscreeny(width : word);
var i : integer;
begin
  for i := 0 to 199 do screen_y[i] := i*width;
end;

procedure init_graph;
var regs : registers;
begin
  regs.ah := $00;
  regs.al := $13;
  intr($10, regs);
  screen.buffer := ptr($a000, 0);
  screen.width := 320;
  calcscreeny(320);
end;

procedure close_graph;
var regs : registers;
begin
  regs.ah := $00;
  regs.al := $03;
  intr($10, regs);
end;

procedure setclipboundary(x1, y1, x2, y2 : integer);
begin
  screen.clip.x_min := x1;
  screen.clip.y_min := y1;
  screen.clip.x_max := x2;
  screen.clip.y_max := y2;
end;

procedure hline(x1, y, x2 : word; color : byte);
var x, address : word;
begin
address := screen_y[y];
  for x := x1 to x2 do screen.buffer^[address+x] := color;
end;

procedure vline(x, y1, y2 : word; color : byte);
var  y, address : word;
begin
  address := screen_y[y1] + x;
  for y := y1 to y2 do
  begin
    screen.buffer^[address] := color;
    address := address + screen.width;
  end;
end;

procedure set_pixel(x, y : word; color : byte);
var
  address : word;
begin
  address := (y shl 8) + (y shl 6) + x;
  screen.buffer^[address] := color;
end;

procedure line(x1, y1, x2, y2 : word; color : byte);
var deltax, deltay, inc1, inc2, d, x, y : integer;
begin
  deltax := x2 - x1; deltay := y2 - y1;
  d := 2 * deltay - deltax;
  inc1 := deltay shl 1;
  inc2 := (deltay-deltax) shl 1;
  x := x1; y := y1;
  set_pixel(x, y, color);
  while x < x2 do
  begin
    if d < x1 then
    begin
      d := d + inc1;
      inc(x);
    end
    else
    begin
      d := d + inc2;
      inc(x);
      inc(y);
    end;
    set_pixel(x, y, color);
  end;
end;

procedure linec(x1, y1, x2, y2 : integer; color : byte);
const
  codebottom = 1;
  codetop    = 2;
  codeleft   = 4;
  coderight  = 8;

function compoutcode(x, y : integer) : byte;
var code : byte;
begin
  code := 0;
  if      y > screen.clip.y_max then code := codebottom
  else if y < screen.clip.y_min then code := codetop;
  if      x > screen.clip.x_max then code := code+coderight
  else if x < screen.clip.x_min then code := code+codeleft;
  compoutcode := code;
end;

var
  outcode0, outcode1, outcodeout : byte;
  x, y : integer;
begin
  outcode0 := compoutcode(x1, y1);
  outcode1 := compoutcode(x2, y2);

  while (outcode0 <> 0) or (outcode1 <> 0) do
  begin
    if (outcode0 and outcode1) <> 0 then
      exit
    else
    begin
      if outcode0 > 0 then
        outcodeout := outcode0
        else
        outcodeout := outcode1;

      if (outcodeout and codebottom) = codebottom then
      begin
        y := screen.clip.y_max;
        x := x1 + longint(x2 - x1) * longint(y - y1) div (y2 - y1);
      end
      else if (outcodeout and codetop) = codetop then
      begin
        y := screen.clip.y_min;
        x := x1 + longint(x2 - x1) * longint(y - y1) div (y2 - y1);
      end
      else if (outcodeout and coderight) = coderight then
      begin
        x := screen.clip.x_max;
        y := y1 + longint(y2 - y1) * longint(x - x1) div (x2 - x1);
      end
      else if (outcodeout and codeleft) = codeleft then
      begin
        x := screen.clip.x_min;
        y := y1 + longint(y2 - y1) * longint(x - x1) div (x2 - x1);
      end;

      if (outcodeout = outcode0) then
      begin
        x1 := x; y1 := y;
        outcode0 := compoutcode(x1, y1);
      end
      else
      begin
        x2 := x; y2 := y;
        outcode1 := compoutcode(x2, y2);
      end;
    end;
  end;

  line(x1, y1, x2, y2, color);
end;

begin
  randomize;

  init_graph;

  setclipboundary(30, 30, 190, 150);

  vline(30, 30, 150, 7);
  vline(190, 30, 150, 7);
  hline(30, 30, 190, 7);
  hline(30, 150, 190, 7);
readln;

  repeat
    x1 := random(320);
    y1 := random(200);
    x2 := random(320);
    y2 := random(200);

if x1 > x2 then
begin
tmp := x2; x2 := x1; x1 := tmp
end;

if y1 > y2 then
begin
tmp := y2; y2 := y1; y1 := tmp
end;

    linec(x1, y1, x2, y2, 7);
  until readkey = #27;

  close_graph;
end.
