unit sbfm;

interface

uses crt;

var 
	port_num : word;

const
	Left_Stereo = $0220;
	Right_Stereo = $0222;
	Mono = $0388;
	o1 = $00;
	o2 = $04;
	o3 = $08;
	o4 = $0C;
	o5 = $10;
	o6 = $14;
	o7 = $18;
	o8 = $1C;
	key_on = $20;
	key_off = $00;
	f_most1 = $0100;
	f_most2 = $0200;
	note_do_diez = $6B + f_most1;
	note_re = $81 + f_most1;
	note_re_diez = $98 + f_most1;
	note_mi = $B0 + f_most1;
	note_fa = $CA + f_most1;
	note_fa_diez = $E5 + f_most1;
	note_sol = $02 + f_most2;
	note_sol_diez = $20 + f_most2;
	note_la = $41 + f_most2;
	note_la_diez = $63 + f_most2;
	note_si = $87 + f_most2;
	note_do = $AE + f_most2;

procedure port_init(port_num : Word);
procedure play_note(port_num, octav, note, duration : Word);
procedure play_note_mono(octav, note, duration : Word);
procedure play_note_stereo(octav, note, duration : Word);
procedure play_pause(duration : word);
procedure sound_on(port_num, freq : Word);
procedure set_modulator_mult(port_num : Word; mult : Byte);
procedure set_modulator_level(port_num : Word; level : Byte);
procedure set_modulator_attack(port_num : Word; attack : Byte);
procedure set_modulator_sustain(port_num : Word; sustain : Byte);
procedure set_carrier_mult(port_num : Word; mult : Byte);
procedure set_carrier_volume(port_num : Word; volume : Byte);
procedure set_carrier_attack(port_num : Word; attack : Byte);
procedure set_carrier_sustain(port_num : Word; sustain : Byte);
procedure sound_off(port_num : word);

implementation
procedure port_init(port_num : Word);
var
	i, k : byte;
begin
	port[port_num] := $01;
	port[port_num] := $00;
	for i := $01 to $F5 do
	begin
		port[port_num] := i;
		port[port_num + $0001] := $00;
	end;
end;

procedure play_note(port_num, octav, note, duration : Word);
begin
	port[port_num] := $A0;
	port[port_num + $0001] := Lo(note);
	port[port_num] := $B0;
	port[port_num + $0001] := key_on + octav + Hi(note);
	delay(duration);
	port[port_num] := $B0;
	port[port_num + $0001] := key_off + octav + Hi(note);
end;

procedure play_note_mono(octav, note, duration : Word);
begin
	port[mono] := $A0;
	port[mono + $0001] := Lo(note);
	port[mono] := $B0;
	port[mono + $0001] := key_on + octav + Hi(note);
	delay(duration);
	port[mono] := $B0;
	port[mono + $0001] := key_off + octav + Hi(note);
end;

procedure play_note_stereo(octav, note, duration : Word);
begin
	port[Left_Stereo] := $A0;
	port[Left_Stereo + $0001] := Lo(note);
	port[Right_Stereo] := $A0;
	port[Right_Stereo + $0001] := Lo(note);
	port[Left_Stereo] := $B0;
	port[Left_Stereo + $0001] := key_on + octav + Hi(note);
	port[Right_Stereo] := $B0;
	port[Right_Stereo + $0001] := key_on + octav + Hi(note);
	delay(duration);
	port[Left_Stereo] := $B0;
	port[Left_Stereo + $0001] := key_off + octav + Hi(note);
	port[Right_Stereo] := $B0;
	port[Right_Stereo + $0001] := key_off + octav + Hi(note);
end;

procedure play_pause(duration : word);
begin
	delay(duration);
end;

procedure sound_on(port_num, freq : Word);
begin
	port[port_num] := $A0;
	port[port_num + $0001] := freq;
	port[port_num] := $B0;
	port[port_num + $0001] := $31;
end;

procedure sound_off(port_num : word);
begin
	port[port_num] := $B0;
	port[port_num + $0001] := $11;
end;

procedure set_modulator_mult(port_num : Word; mult : Byte);
begin
	port[port_num] := $20;
	port[port_num + $0001] := mult;
end;

procedure set_modulator_level(port_num : Word; level : Byte);
begin
	port[port_num] := $40;
	port[port_num + $0001] := level;
end;

procedure set_modulator_attack(port_num : Word; attack : Byte);
begin
	port[port_num] := $60;
	port[port_num + $0001] := attack;
end;

procedure set_modulator_sustain(port_num : Word; sustain : Byte);
begin
	port[port_num] := $80;
	port[port_num + $0001] := sustain;
end;

procedure set_carrier_mult(port_num : Word; mult : Byte);
begin
	port[port_num] := $23;
	port[port_num + $0001] := mult;
end;

procedure set_carrier_volume(port_num : Word; volume : Byte);
begin
	port[port_num] := $43;
	port[port_num + $0001] := volume;
end;

procedure set_carrier_attack(port_num : Word; attack : Byte);
begin
	port[port_num] := $63;
	port[port_num + $0001] := attack;
end;

procedure set_carrier_sustain(port_num : Word; sustain : Byte);
begin
	port[port_num] := $83;
	port[port_num + $0001] := sustain;
end;

begin
end.
