{$r+}
unit matrices;

interface

uses crt;

const 
	max = 3;

type
	str10 = string[10];
	str30 = string[30];
	matrix = array[1..max, 1..max] of pointer;
	rational = record  
		num, den : longint;  
	end;
	rational_ptr = ^rational;
	rational_matrix = array[1..max, 1..max] of rational;
	real_ptr = ^real;
	real_matrix = array[1..max, 1..max] of real;
	complex = record
		re, im : real; 
	end;
	complex_ptr = ^complex;
	complex_matrix = array[1..max, 1..max] of complex;

	matrix_obj = object
		M : matrix;
		constructor init;
		procedure set_to_zero(var  p : pointer); virtual;
		procedure scalar_sum(const p, q : pointer;
			var r : pointer); virtual;
		procedure scalar_product(const p, q : pointer;
			var r : pointer); virtual;
		procedure matrix_sum(const A, B : matrix);
		procedure matrix_product(const A, B : matrix);
		procedure convert_to_string(p : pointer; var tt : str30); virtual;
		procedure print(name : str10);
		destructor done;
	end;

	real_matrix_obj = object(matrix_obj)
		constructor init(M0 : real_matrix);
		procedure set_to_zero(var p : pointer); virtual;
		procedure scalar_sum(const p, q : pointer; 
			var r : pointer); virtual;
		procedure scalar_product(const  p, q : pointer;
			var r : pointer); virtual;
		procedure convert_to_string(p : pointer; 
			var tt : str30); virtual;
		destructor done;
	end;

	rational_matrix_obj = object(matrix_obj)
		constructor init(M0 : rational_matrix);
		procedure set_to_zero(var p : pointer); virtual;
		procedure scalar_sum(const p, q : pointer;
			var r : pointer); virtual;
		procedure scalar_product(const p, q : pointer;
			var r : pointer); virtual;
		procedure convert_to_string(p : pointer;
			var  tt : str30); virtual;
		destructor done;
	end;

	complex_matrix_obj = object(matrix_obj)
		constructor init(M0 : complex_matrix);
		procedure set_to_zero(var p : pointer); virtual;
		procedure scalar_sum(const p, q : pointer;
			var r : pointer); virtual;
		procedure scalar_product(const p, q : pointer;
			var r : pointer); virtual;
		procedure convert_to_string(p : pointer;
			var tt : str30);  virtual;
		destructor  done;
	end;

implementation

constructor matrix_obj.init;
begin
end;

procedure matrix_obj.set_to_zero(var p : pointer);
begin
end;

procedure matrix_obj.scalar_sum(const p, q : pointer; var r : pointer);
begin
end;

procedure matrix_obj.scalar_product(const p, q : pointer; var r : pointer);
begin
end;

procedure matrix_obj.matrix_sum(const A, B : matrix);
var 
	i, j : word;
begin
	for i := 1 to max do
		for j := 1 to max do
			scalar_sum(A[i, j], B[i, j], M[i, j]);
end;

procedure matrix_obj.matrix_product(const A, B : matrix);
var 
	i, j, k : word;
	prod, sum : pointer;
begin
	getmem(prod, SizeOf(M[1, 1]));
	for i := 1 to max do
		for k := 1 to max do
		begin
			sum := M[i, k];
			set_to_zero(sum);
			for j := 1 to max do
			begin
				scalar_product(A[i, j], B[j, k], prod);
				scalar_sum(sum, prod, sum);
			end;
	end;
	freemem(prod, SizeOf(M[1, 1]));
end;

procedure matrix_obj.convert_to_string(p : pointer; var tt : str30);
begin
end;

procedure matrix_obj.print(name : str10);
var 
	i, j, k, hold_y : word;
	temp : str30;
begin
	hold_y := wherey;
	gotoxy(1, hold_y + (max + 1) div 2);
	write(name, ' = ');
	k := length(name) + 4;
	gotoxy(k, hold_y);  write('');
	for i := 1 to max do
	begin
		gotoxy(k, hold_y + i);
		write('|');
		for j := 1 to max do
		begin
			convert_to_string(M[i, j], temp);
			write(temp);
		end;
		write('|');
	end;
	gotoxy(k, max + hold_y + 1);  write('');
	gotoxy(k + length(temp) * max + 1, hold_y);
	write('');
	gotoxy(k + length(temp) * max + 1, max + hold_y + 1);
	writeln('');
end;

destructor matrix_obj.done;
begin
end;

constructor real_matrix_obj.init(M0 : real_matrix);
var  
	i, j : word;
begin
	inherited init;
	for i := 1 to max do
		for j := 1 to max do
		begin
			new(real_ptr(M[i, j]));
			real_ptr(M[i, j])^ := M0[i, j];
		end;
end;

procedure real_matrix_obj.set_to_zero(var p : pointer);
begin
	real_ptr(p)^ := 0.0;
end;

procedure real_matrix_obj.scalar_sum(const p, q : pointer; var r : pointer);
begin
	if not Assigned(r) then 
		new(real_ptr(r));
	real_ptr(r)^ := real_ptr(p)^ + real_ptr(q)^;
end;

procedure real_matrix_obj.scalar_product(const p, q : pointer; 
		var
			r : pointer);
begin
	if not Assigned(r) then 
		new(real_ptr(r));
	real_ptr(r)^ := real_ptr(p)^ * real_ptr(q)^;
end;

procedure real_matrix_obj.convert_to_string(p : pointer; var tt : str30);
begin
	str(real_ptr(p)^:9:4, tt);
end;

destructor real_matrix_obj.done;
var 
	i, j : word;
begin
	for i := 1 to max do
		for j := 1 to max do
		begin
			dispose(real_ptr(M[i, j]));
		end;
	inherited done;
end;

constructor rational_matrix_obj.init(M0 : rational_matrix);
var 
	i, j : word;
begin
	inherited init;
	for i := 1 to max do
		for j := 1 to max do
		begin
			new(rational_ptr(M[i, j]));
			rational_ptr(M[i, j])^ := M0[i, j];
		end;
end;

procedure rational_matrix_obj.set_to_zero(var p : pointer);
begin
	rational_ptr(p)^.num := 0;
	rational_ptr(p)^.den := 1;
end;

procedure lowest_terms(var x, y : longint);
var 
	a, b, r : longint;
begin
	a := x;  
	b := y;
	a := Abs(a);
	repeat
		r := a mod b;  
		a := b;  
		b := r;
	until r = 0;
	x := x div a;  
	y := y div a;
end;

procedure rational_matrix_obj.scalar_sum(const p, q : pointer; var
			r : pointer);
var 
	x, y, z : rational;
begin
	x := rational_ptr(p)^;
	y := rational_ptr(q)^;
	z.num := x.num * y.den + y.num * x.den;
	z.den := x.den * y.den;
	lowest_terms(z.num, z.den);
	if  not Assigned(r)  then  
		new(rational_ptr(r));
	rational_ptr(r)^ := z;
end;

procedure rational_matrix_obj.scalar_product(const p, q : pointer; var r : pointer);
var 
	x, y, z : rational;
begin
	x := rational_ptr(p)^;
	y := rational_ptr(q)^;
	z.num := x.num * y.num;
	z.den := x.den * y.den;
	lowest_terms(z.num, z.den);
	if not Assigned(r) then 
		new(rational_ptr(r));
	rational_ptr(r)^ := z;
end;

procedure rational_matrix_obj.convert_to_string(p : pointer; var tt : str30);
var 
	t1, t2 : string[11];
	j : word;
begin
	str(rational_ptr(p)^.num, t1);
	str(rational_ptr(p)^.den, t2);
	tt := t1 + '/' + t2;
	for j := length(t1) + length(t2) to 12 do tt := ' ' + tt;
end;

destructor rational_matrix_obj.done;
var 
	i, j : word;
begin
	for i := 1 to max do
		for j := 1 to max do
		begin
			dispose(rational_ptr(M[i, j]));
		end;
	inherited done;
end;

constructor complex_matrix_obj.init(M0 : complex_matrix);
var 
	i, j : word;
begin
	inherited init;
	for i := 1 to max do
		for j := 1 to max do
		begin
			new(complex_ptr(M[i, j]));
			complex_ptr(M[i, j])^ := M0[i, j];
		end;
end;

procedure complex_matrix_obj.set_to_zero(var p : pointer);
begin
	complex_ptr(p)^.re := 0.0;
	complex_ptr(p)^.im := 0.0;
end;

procedure complex_matrix_obj.scalar_sum(const p, q : pointer; var r : pointer);
var 
	x, y, z : complex;
begin
	x := complex_ptr(p)^;
	y := complex_ptr(q)^;
	z.re := x.re + y.re;
	z.im := x.im + y.im;
	if r = nil then 
		new(complex_ptr(r));
	complex_ptr(r)^ := z;
end;

procedure complex_matrix_obj.scalar_product(const p, q : pointer; var  r : pointer);
var 
	x, y, z : complex;
begin
	x := complex_ptr(p)^;
	y := complex_ptr(q)^;
	z.re := x.re * y.re - x.im * y.im;
	z.im := x.re * y.im + x.im * y.re;
	if not Assigned(r) then 
		new(complex_ptr(r));
	complex_ptr(r)^ := z;
end;

procedure complex_matrix_obj.convert_to_string(p : pointer; var tt : str30);
var 
	t1, t2 : string[6];
begin
	str(complex_ptr(p)^.re:6:4, t1);
	str(complex_ptr(p)^.im:6:4, t2);
	tt := '  ' + t1 + ' + ' + t2 + 'i';
end;

destructor complex_matrix_obj.done;
var 
	i, j : word;
begin
	for i := 1 to max do
		for j := 1 to max do
		begin
			dispose(complex_ptr(M[i, j]));
		end;
	inherited done;
end;

end.
