unit series;

interface

const 
	max_deg = 20;
	eta = 1.0e-20;

type 
	power_series = array[0..max_deg] of real;

var 
	Zero, One: power_series;
	k: word;

procedure quotient(const F, G : power_series; var H : power_series);
procedure composition(const F, G : power_series; var H : power_series);
procedure invert(const F : power_series; var H : power_series);
procedure nth_power(const F : power_series; n : word; var G : power_series);

implementation

procedure quotient(const F, G : power_series; var H : power_series);
var 
	r, k : word;
	sum : real;
begin
	if (F[0] <> 1.0) or (G[0] <> 1.0) then
		halt;
	H := One;
	for r := 1 to max_deg do
	begin
		sum := F[r];
		for k := 0 to r - 1 do 
			sum := sum - H[k] * G[r - k];
		H[r] := sum;
	end;
end;


procedure composition(const F, G : power_series; var H : power_series);
var 
	j, k : word;
	P : power_series;
begin
	if (F[0] <> 0.0) or (G[0] <> 0.0) or
 	(F[1] <> 1.0) or (G[1] <> 1.0)  then
			halt;
	for k := 1 to max_deg 
		do H[k] := F[1] * G[k];
	for k := 2 to max_deg do
		if F[k] <> 0.0 then
		begin
			nth_power(G, k, P);
			for j := k to max_deg do 
				H[j] := H[j] + F[k] * P[j];
		end;
end;

procedure invert(const F : power_series; var H : power_series);
var 
	P : power_series;
	k, m : word;

procedure truncated_product(var P : power_series; m : word);
var
	j, k : word;
	sum : real;
begin
	for j := m downto 1 do
	begin
		sum := 0;
		for k := 1 to j do
			sum := sum + P[j - k] * H[k];
		P[j] := sum;
	end;
	P[0] := 0.0;
end;{truncated_product}

begin{invert}
	if (F[0] <> 0.0) or (F[1] <> 1.0) then
		halt;
	H := Zero;
	H[1] := 1.0;
	for m := 2 to max_deg  do
	begin
		P := Zero;
		for k := m downto 2 do
		begin
			P[0] := -F[k];
			truncated_product(P, m);
		end;
		truncated_product(P, m);
		P[1] := P[1] + 1.0;
		H := P;
	end;
end;{invert}

procedure nth_power(const F : power_series; n : word; var G : power_series);
var
	deg_F, deg_G, j, k, r : integer;
	p, sum, t : real;
begin
	if F[0] <> 0.0 then
		exit;
	G := Zero;
	deg_F := 0;
	repeat
		inc(deg_F)
	until (abs(F[deg_F]) >= eta) or (deg_F = max_deg);
	deg_G := deg_F * n;
	if (F[deg_F] = 0.0) or (deg_G > max_deg)  then
		exit;
	p := 1.0;
	t := F[deg_F];
	for j := 1 to n do
		p := p * t;
	G[deg_G] := p;
	for r := deg_G + 1 to max_deg do
	begin
		sum := 0.0;
		k := r;
		for j := deg_F + 1  to  r + deg_F - deg_G  do
		begin
			dec(k);
			sum := sum + (k - n * j) * F[j] * G[k];
		end;
		G[r] := - sum / ( (r - n * deg_F) * F[deg_F] );
	end;
end;


begin
	for k := 0 to max_deg do
		Zero[k] := 0.0;
	One := Zero;
	One[0] := 1.0;
end.
	