unit lin_alg;

{ $define complete_pivot }
interface

const  max_no_rows = 20;

type  matrix = array[1..max_no_rows, 1..2*max_no_rows] of real;
       str10 = string[10];

var   no_rows, no_cols: word;
         singular: boolean;

procedure get_system_matrix(var  A: matrix);
procedure get_rhs(var  A: matrix);
procedure print_matrix(const  A: matrix;  name: str10;
                           screen_row, first_col, last_col: word);
procedure triangularize(var  A: matrix);
procedure back_subtract(var  A: matrix);

implementation

uses  crt;

const  eps = 1.0e-10;

var      change: boolean;
        no_vectors: word;
{$ifdef complete_pivot}
   var        s, t: word;
        column_rec: array[1..max_no_rows] of word;
{$endif}

   procedure  correct_input(var  A: matrix; first_col, last_col: word);

      var  row, col: word;

      begin
      writeln(' j, k,    A[j, k]   ப १ ஡,');
      writeln('⥬  <Enter>');
      write('j k A[j, k]: ');
      read(row, col);
      if  (row >= 1) and (row <= no_rows) and
          (col >= first_col) and (col <= last_col)  then
             readln(A[row, col]);
      clrscr;
      print_matrix(A, 'A', 1, 1, no_rows);
      end;

   procedure  get_system_matrix(var  A: matrix);

      var  row, col: word;
                 ch: char;

      begin
      clrscr;
      write(' ࠧ  m  1  ', max_no_rows, ': ');
      readln(no_rows);
      writeln;
      writeln('  ', no_rows, ' x ', no_rows, '  A');
      writeln('  ப  ࠧ ');
      writeln('१ ஡, ⥬  <Enter>');
      writeln;
      for  row := 1  to  no_rows  do
         begin
         write('A[', row, ', 1] ... A[' , row, ', ', no_rows, ']: ');
         for  col := 1  to  no_rows  do  read(A[row, col]);
         readln;
         writeln;
         end;
      clrscr;
      writeln(' :');
      print_matrix(A, 'A', 2, 1, no_rows);
      change := false;
      write('ࠢ쭮?  Y(es?  N(o?  ');
      repeat
         ch := readkey;  writeln;
         if  upcase(ch) = 'N'  then
            begin
            change := true;
            correct_input(A, 1, no_rows);
            write('  ?  Y(es?  N(o?  ');
            end;
      until  upcase(ch) <> 'Y';
      if  change  then
         begin
         clrscr;
         print_matrix(A, 'A', 1, 1, no_rows);
         end;
      end;

   procedure  get_rhs(var  A: matrix);

      var  row, col: word;
                 ch: char;

      begin
      writeln;
      write('  ?');
      write('  Y(es ?  N(o ?  ');  ch := readkey;  writeln;
      if  upcase(ch) = 'Y'  then
         begin
         no_vectors := no_rows;
         no_cols := 2 * no_rows;
         for  row := 1  to  no_rows  do
            for  col := no_rows + 1  to  no_cols  do  A[row, col] := 0;
         for  row := 1  to  no_rows  do  A[row, no_rows + row] := 1;
         exit;
         end;
      writeln;
      write('᫮ ⮫殢 ࠢ  (  ', no_rows, '): ');
      readln(no_vectors);
      writeln;
      no_cols := no_rows + no_vectors;
      writeln;
      writeln(' ', no_vectors, ' ⮫殢 ࠢ :');
      writeln('    ப, ');
      writeln('१ ஡, ⥬  <Enter>');
      for  row := 1  to  no_rows  do
         begin
         write('rhs[', row, ', 1]..rhs[' , row, ', ', no_vectors, ']: ');
         for  col := no_rows + 1  to  no_cols  do  read(A[row, col]);
         readln;
         end;
      clrscr;
      print_matrix(A, 'rhs', 1, no_rows + 1, no_cols);
      change := false;
      write('ࠢ쭮?  Y(es?  N(o?  ');
      repeat
         ch := readkey;  writeln;
         if  upcase(ch) = 'N'  then
            begin
            change := true;
            correct_input(A, no_rows + 1, no_cols);
            write('ࠢ쭮?  Y(es?  N(o?  ');
            end;
      until  upcase(ch) <> 'Y';
      if  change  then
         begin
         clrscr;
         print_matrix(A, 'rhs', 1, no_rows + 1, no_cols);
         end;
      end;

   procedure  print_matrix(const  A: matrix;  name: str10;
                           screen_row, first_col, last_col: word);

      var  j, k, l, m: word;

      begin
      m := 10 * (last_col - first_col + 1) + 1;
      gotoxy(1, screen_row + (no_rows + 1) div 2);
      write(name, ' = ');
      l := length(name) + 4;
      gotoxy(l, screen_row);  write('');
      gotoxy(l + m, screen_row);
      write('');
      for  j := 1  to  no_rows  do
         begin
         gotoxy(l, screen_row + j);
         write('');
         for  k := first_col  to  last_col  do  write(A[j, k]:10:2);
         write('');
         end;
      gotoxy(l, screen_row + no_rows + 1);  write('');
      gotoxy(l + m, screen_row + no_rows + 1);
      writeln('');
      end;

   procedure  subtract_rows(var  A: matrix;  p, q: word;  x: real);

      var  j: word;

      begin
      for  j := p + 1  to  no_cols  do
         A[q, j] := A[q, j] - x * A[p, j];
      end;

    {$ifdef complete_pivot}
    procedure  change_columns(var A : matrix; p, t: word);
       var  j: word;  x: real;

       begin
       for  j := 1  to  no_rows  do
          begin
          x := A[j, p];
          A[j, p] := A[j, t];
          A[j, t] := x;
          end;
       end;
      {$endif}

   procedure  triangularize(var  A: matrix);

      var  p, q, k: word;

      function  partial_pivot(p: word): word;

         var  j, q: word;
              x, y: real;

         begin
         q := p;  x := abs(A[p, p]);
         for  j := p + 1  to  no_rows  do
            begin
            y := abs(A[j, p]);
            if  y > x  then  begin  q := j;  x := y  end;
            end;
         singular := (abs(x) < eps);
         partial_pivot := q;
         end;

      procedure  change_rows(p, q: word);
         var  j: word;
              x: real;

         begin
         for  j := p  to  no_cols  do
            begin
            x := A[p, j];
            A[p, j] := A[q, j];
            A[q, j] := x;
            end
         end;

      {$ifdef complete_pivot}
      procedure  find_complete_pivot(p: word;  var  s, t: word);
         var  j, k: word;
              x, y: real;

         begin
         s := p;  t := p;  x := abs(A[p, p]);
         for  j := p  to  no_rows  do
            for  k := p  to  no_rows  do
               begin
               y := abs(A[j, k]);
               if  y > x  then
                  begin  x := y;  s := j;  t := k;  end;
               end;
         singular := (abs(x) < eps);
         end;
      {$endif}

      begin
      p := 1;
      repeat
         {$ifdef complete_pivot}
         find_complete_pivot(p, s, t);
         if  not singular  then
            begin
            if  s > p  then  change_rows(p, s);
            if  t > p  then
               begin
               transpose_columns(A, p, t);
               column_rec[p] := t;
               end
            else  column_rec[p] := p;
         {$else}
         k := partial_pivot(p);
         if  not singular  then
            begin
            if  k > p  then  change_rows(p, k);
         {$endif}
            for  q := p + 1  to  no_rows  do
               subtract_rows(A, p, q, A[q, p] / A[p, p]);
            end;
         inc(p);
      until  (p > no_rows) or singular;
      end;

   procedure  back_subtract(var  A: matrix);

      var  p, q:  word;

      procedure  normalize_row(p: word);
         var  k: word;
              x: real;

         begin
         x := A[p, p];  A[p, p] := 1.0;
         for  k := p + 1  to  no_cols  do
            A[p, k] := A[p, k] / x;
         end;

      {$ifdef complete_pivot}
      procedure  restore_col_order;

         var  j, k: word;  x: real;

         begin
         for  j := no_rows - 1  downto  1  do
            begin
            p := column_rec[J];
            if  p > j  then
               begin
               for  k := no_rows + 1  to  no_cols  do
                  begin
                  x := A[j, k];  A[j, k] := A[p, k];  A[p, k] := x;
                  end;
               end;
            end;
         end;
      {$endif}

      begin
      for  p := 1  to  no_rows  do  normalize_row(p);
      for  p := no_rows  downto  2  do
  for  q := p - 1  downto  1  do subtract_rows(A, p, q, A[q, p]);
      {$ifdef complete_pivot}
      restore_col_order;
      {$endif}
      end;


end.