unit  symm_mat;

interface

const  max_dim = 50;

type  symmetric_matrix = array[1..max_dim, 1..max_dim] of real;
                   vector = array[0..max_dim] of real;
                     str4 = string[4];

   var  dim: word;

   procedure  get_matrix(var  A: symmetric_matrix);
   procedure  print_matrix(var  A: symmetric_matrix;  name: str4);
   procedure  iterate(const  A: symmetric_matrix;  var  v: vector;
                      var  r: real;  var  fail: boolean);
   procedure  QR_algorithm(var  A: symmetric_matrix;
        var  char_root: vector; var  W: symmetric_matrix);

implementation

   uses  crt;

   const  tol = 1.0e-30;

   procedure  get_matrix(var  A: symmetric_matrix);

      var  row, col: word;
                 ch: char;
             change: boolean;

      procedure  correct_input;

         var  row, col: integer;

         begin
         writeln('Enter j, k and A[j, k] on one line, separated by spaces.');
         writeln('Then press <Enter>: ');
         write('j k A[j, k]: ');
         read(row, col);
         if  (row >= 1) and (row <= dim) and
             (col >= 1) and (col <= dim)  then  readln(A[row, col]);
         clrscr;
         print_matrix(A, 'A');
         end;

      begin
      clrscr;
      write(' ࠧ  n  1  ', max_dim, ': ');
      readln(dim);  writeln;
      writeln(' ', dim, ' x ', dim, ' ᨬ  A');
      writeln('  -   ப १ ஡');
      writeln('⥬  <Enter>');
      writeln;
      for  row := 1  to  dim  do
         begin
         write('A[', row, ', ', row, '] ... A[' , row, ', ', dim, ']: ');
         for  col := 1  to  dim  do
            begin
            read(A[row, col]);
            A[col, row] := A[row, col];
            end;
         readln;  writeln;
         end;
      writeln(' :');
      print_matrix(A, 'A');  change := false;
      write('ࠢ쭮?  Y(es?  N(o?  ');
      repeat
         ch := readkey;  writeln;
         if  upcase(ch) = 'N'  then
            begin
            change := true;
            correct_input;
            write('ࠢ?  Y(es?  N(o?  ');
            end;
      until  upcase(ch) <> 'Y';
      if  change  then
         begin
         clrscr;
         print_matrix(A, 'A');
         end;
      end;

   procedure  print_matrix(var  A: symmetric_matrix;  name: str4);

      var  row, col, l: word;

      begin
      clrscr;
      gotoxy(1, (dim + 1) div 2);
      write(name, ' = ');
      l := length(name) + 4;
      for  row := 1  to  dim  do
         begin
         gotoxy(l, row);
         if  row = 1  then  write('')
         else  if  row = dim  then  write('')  else  write('');
         for  col := 1  to  dim  do  write(A[row, col]:10:5);
         if  row = 1  then   write('')
         else  if  row = dim  then  writeln('')  else  write('');
         end;
      end;

   procedure  iterate(const  A: symmetric_matrix;  var  v: vector;
                      var  r: real;  var  fail: boolean);

      var            x: real;
           count, j, k: word;
                     w: vector;
                  test: boolean;

      function  norm_squared(const  v: vector): real;

         var  x: real;
              j: integer;

         begin
         x := 0.0;
         for  j := 1  to  dim  do  x := x + sqr(v[j]);
         norm_squared := x;
         end;

      function  distance_squared(const  v, w: vector): real;

         var  x: real;
              j: integer;

         begin
         x := 0.0;
         for  j := 1  to  dim  do
            x := x + sqr(v[j] - w[j]);
         distance_squared := x;
         end;

      begin
      count := 0;  fail := false;  test := false;
      repeat
         inc(count);
         w := v;
         for  j := 1  to  dim  do
            begin   
            x := 0.0;
            for  k := 1  to  dim  do  x := x + A[k, j]*w[k];
            v[j] := x;
            end;
         x := norm_squared(v);
         if  x < tol  then  fail := true
         else
            begin
            r := 0.0;
            for  j := 1  to  dim  do  r := r + v[j]*w[j];
            x := 1.0/sqrt(x);
            if  r < 0.0  then  x := -x;
            for  j := 1  to  dim  do  v[j] := x*v[j];
            test := (distance_squared(v, w) < tol);
            end;
      until  fail or test or (count = 200);
      fail := not test;
      end;

   procedure  QR_algorithm(var  A: symmetric_matrix;
      var  char_root: vector; var  W: symmetric_matrix);

      var     diagonal, sine, cosine,
            super_diag, super_diag2: vector;
             norm, eps: real;
                  m, n: word;
                    A0: symmetric_matrix;
           c, s, x, x1, y, z, u, lambda, mu: real;

      procedure  initialize_QR;

         var  j, k: word;

         procedure  tri_diagonalize(var  A, W: symmetric_matrix;
             var  diagonal, super_diag: vector);

            var  c, t, x, y: real;
                          v: vector;
                    i, j, k: word;

            procedure  Householder_step(i: word);

               var  p: vector;
                    j, k: word;
                    z: real;

               begin
               for  j := i + 1  to  dim  do
                  begin
                  t := 0.0;
                  for  k := i + 1  to  dim  do
                     t := t + A[j, k] * v[k];
                  p[j] := 2.0 * t;
                  end;
               z := 0.0;
               for  j := i + 1 to  dim  do
                  z := z + p[j] * v[j];
               z := 2.0 * z;
               for  j := i + 1  to  dim  do
                  for  k := j  to  dim  do
              A[j, k] := A[j, k] - v[j] * p[k] - (p[j] - z * v[j]) * v[k];
               end;

            procedure  update_W(i: word);

               var  p: vector;
                    j, k: word;

               begin
               for  j := 1  to  dim  do
                  begin
                  t := 0.0;
                  for  k := i + 1  to  dim  do
                     t := t + W[j, k]*v[k];
                  p[j] := 2.0*t;
                  end;   
               for  j := 1  to  dim  do
                  for  k := i + 1  to  dim  do
                     W[j, k] := W[j, k] - p[j]*v[k];
               end;

               begin
               super_diag[dim] := 0.0;
               for  i := 1  to  dim - 2  do
                  begin
                  diagonal[i] := A[i, i];
                  t := 0.0;
                  for  j := i + 1  to  dim  do  t := t + sqr(A[i, j]);
                  c := sqrt(t);  x := A[i, i + 1];
                  if  x > 0.0  then  c := -c;
                  super_diag[i] := c;
                  if  c <> 0.0  then
                     begin
                     y := sqrt(0.5*(1.0 - x/c));
                     v[i + 1] := y;
                     y := -1.0/(2.0*c*y);
                     for  j := i + 2  to  dim  do  v[j] := y*A[i, j];
                     Householder_step(i);
                     update_W(i);
                     end;
                  end;
               k := dim - 1;
               diagonal[k] := A[k, k];
               diagonal[dim] := A[dim, dim];
               super_diag[k] := A[k, dim];
               super_diag[dim] := 0.0;
               end;

         begin
         for  j := 1  to  dim  do
            begin
            for  k := 1  to  dim  do  W[j, k] := 0.0;
            W[j, j] := 1.0;
            end;
         tri_diagonalize(A, W, diagonal, super_diag);
         norm := 0.0;  y := 0.0;
         for  j := 1  to  dim  do
            begin
            x := y + abs(diagonal[j]);
            y := abs(super_diag[j]);
            x := x + y;
            if  x > norm  then  norm := x;
            end;
         eps := norm * tol;
         for  j := 0  to  dim  do
            begin  sine[j] := 0.0;  cosine[j] := 0.0;  end;
         end;

  procedure  translate(a, b, c: real; var  lambda, mu: real);

         var  trace, det, lambda0, b2, srt: real;

         begin
         b2 := sqr(b);  det := a*c - b2;  trace := a + c;
         srt := sqrt(sqr(trace) - 4.0*det);
         if  trace >= 0.0  then  lambda := trace + srt
         else  lambda := trace - srt;
         lambda := 0.5 * lambda;
         lambda0 := det / lambda;
         if  abs(lambda0 - mu) < 0.5 * abs(lambda0)  then
            begin  lambda := lambda0;  mu := lambda0  end
         else  if  abs(lambda - mu) < 0.5*abs(lambda)  then
            mu := lambda
         else  begin  mu := lambda0;  lambda := 0.0;  end;
         end;

      procedure  factor(m, n: word);
         var  j: word;

         begin
         diagonal[m] := diagonal[m] - lambda;
         u := super_diag[m];
         for  j := m  to  n - 1  do
            begin
            x := diagonal[j];
            x1 := diagonal[j + 1] - lambda;
            y := super_diag[j];
            z := sqrt(sqr(x) + sqr(u));
            c := x/z;  s := u/z;
            cosine[j] := c;  sine[j]:= s;
            diagonal[j] := z;
            diagonal[j + 1] := -s*y + c*x1;
            super_diag[j] :=    c*y + s*x1;
            u := super_diag[j + 1];
            super_diag[j + 1] := c*u;
            super_diag2[j] := s*u;
            end;
         end;

      procedure  multiply(m, n: word);
         var  j, k: word;

         begin
         for  k := m  to  n - 1  do
            begin
            c := cosine[k];  s := sine[k];
            x := diagonal[k];  y := super_diag[k];
            super_diag[k - 1] := super_diag[k - 1]*c
                                  + super_diag2[k - 1]*s;
            diagonal[k] :=    x*c + y*s + lambda;
            super_diag[k] := -x*s + y*c;
            diagonal[k + 1] := diagonal[k + 1]*c;
            for  j := 1  to  dim  do
               begin
               x := W[j, k];  y := W[j, k + 1];
               W[j, k] :=      x*c + y*s;
               W[j, k + 1] := -x*s + y*c;
               end;
            end;
         diagonal[n] := diagonal[n] + lambda;
         end;

      procedure  report_result;

         var  j, k: word;

         begin
         writeln(':');
         for  j := 1  to  dim  do
            begin
            writeln('r_', j, ' = ',  char_root[j]:11:7);
            write('v_', j, ' = [');
            for  k := 1  to  dim  do
               begin
               write(W[k, j]:10:7);
               if  k <  dim  then  write(', ')
               else  writeln(']''');
               end;
            end;
         writeln;
         end;

      begin
      A0 := A;
      initialize_QR;
      n := dim;
      mu := 0.0;
      while  n > 0  do
         begin
         if  abs(super_diag[n - 1]) < eps  then
            begin
            char_root[n] := diagonal[n];
            dec(n);
            end
         else
            begin
            m := n - 1;
            repeat  dec(m)  until  abs(super_diag[m]) < eps;
            inc(m);
            translate(diagonal[n - 1], super_diag[n - 1],
                      diagonal[n], lambda, mu);
            factor(m, n);
            super_diag2[m - 1] := 0.0;
            super_diag[m - 1] := 0.0;
            multiply(m, n);
            end;
         end;
      report_result;
      end;

end.
