program nonlinear_system;

uses crt;

const 
	max = 20;
	eps = 1.0e-10;
	eta = 1.0e-3;

type 
	vector_fun = function(j : word; x, y : real) : real;

var
	x, y : real;
	i, n : word;
	ch : char;

function F(x, y : real) : real;
begin
	F := cos(x) * cos(y) - 0.1 * x - 0.2 * y + 0.15 * x * y;
end;


function G(j : word;  x, y : real) : real; far;
begin
	if j = 1 then
		G := -sin(x) * cos(y) - 0.1 + 0.15 * y
	else
		G := -cos(x) * sin(y) - 0.2 + 0.15 * x;
end;

function dG(j, k : word;  x, y : real) :  real; far;
begin
	if j = 1 then
	begin
		if  k = 1  then
			dG := -cos(x) * cos(y)
		else  
			dG := sin(x) * sin(y) + 0.15;
	end
	else
	begin
		if k = 1  then
			dG := sin(x) * sin(y) + 0.15
		else  
			dG := -cos(x) * cos(y);
	end;
end;


procedure solve(G : vector_fun;  var  x, y : real);
var
	j, k : integer;
	r, s, t, u, v : real;
	singular : boolean;
	ch : char;

procedure  Newton_step(var  x, y : real);
var
	det : real;
	j, k : word;
begin
	u := G(1, x, y);
	v := G(2, x, y);
	r := dG(1, 1, x, y);
	s := dG(1, 2, x, y);
	t := dG(2, 2, x, y);
	det := r * t - s * s;
	if  abs(det) < eta  then
		singular := true
	else
	begin
		singular := false;
		x := x - (t * u - s * v) / det;
		y := y - (-s * u + r * v) / det;
	end
end;{Newton_step}

begin{solve}
	repeat
		Newton_step(x, y);
		dec(i);
	until singular or (i = 0) or ( abs(u) + abs(v) < eps );
	if  singular  then
		writeln('襭 ')
	else
		if  i = 0  then
		begin
			writeln(n,' 権');
			writeln('F(', x, ', ', y, ') = (', u, ', ', v, ')')
		end
		else
			writeln('襭: (x, y) = (', x, ', ', y, ')');
end;{solve}

begin
	clrscr;
	x := 0.0;
	y := 0.0;
	i := max;
	solve(G, x, y);
	writeln('F(x, y) =', F(x, y));
	writeln;
	write(' <Enter>');  readln;
end.





	