program shoot;

uses crt, rkutta;

const
	u26 = 26.0 * 5.5e-20;
	eps = 5.0e-15;
	eta = 5.0e-15;

var
	delta_t : real;
	x0, x1 : vector;
	fun_count, k : word;

procedure F(const  t : real;  const  x : vector; var  w : vector); far;
begin
	w[1] := x[2];
	w[2] := -1 - (1 + sqr(t)) * x[1];
end;

procedure solve_ode(dim : word; const  t0, t1 : real; const  x0 : 
		vector; var x1 : vector);
const
	max_calls = 20000;
var
	dt, min_dt, t, tol, temp,
	norm_error : real;
	current_x, next_x, error, y1, der_x : vector;
	output_count : word;
	advanced, done : boolean;

procedure  initialize_dt;
var
	norm_der_x : real;
begin
	fun_count := 0;
	output_count := 0;
	t := t0;
	delta_t := t1 - t0;
	current_x := x0;
	F(t, current_x, der_x);
	tol := eta * sup_norm(dim, current_x) + eps;
	dt := abs(delta_t);
	norm_der_x := sup_norm(dim, der_x);
	if  tol < norm_der_x * x2y(dt, 4.0)  then
	begin
		dt := u26 * max(abs(t), dt);
		dt := max(dt, x2y(tol / norm_der_x, 0.25));
	end;
	if  delta_t < 0  then
		dt := -dt;
end;{initialize_dt}

begin{solve_ode}
	initialize_dt;
	repeat
		if  abs(dt) > 2.0 * abs(delta_t) then 
			inc(output_count);
		if  abs(delta_t) <= u26 * abs(t)  then
		begin
			for k := 1 to dim do 
				x1[k] := current_x[k] + delta_t * der_x[k];
			exit;
		end;
		done := false;
		advanced := true;
		min_dt := u26 * abs(t);
		delta_t := t1 - t;
		if  abs(delta_t) < 2.0 * abs(dt)  then
		begin
			if  0.9 * abs(delta_t) > abs(dt)  then
				dt := 0.5 * delta_t
			else
			begin
				done := true;
				dt := delta_t;
			end;
		end;
		repeat
			KM34(dim, F, t, dt, current_x, next_x, y1, error);
			tol := sup_norm(dim, current_x) + sup_norm(dim, next_x);
			tol := 0.5 * tol * eta + eps;
			norm_error := sup_norm(dim, error);
			if  norm_error >= tol  then
			begin
				advanced := false;
				done := false;
				if norm_error >= 6561.0 * tol then
					dt := 0.1 * dt
				else 
					dt := 0.9 * dt / x2y(norm_error / tol, 0.25);
			end;
		until  norm_error < tol;
		t := t + dt;
		current_x := next_x;
		F(t, current_x, der_x);
		inc(fun_count);
		if  advanced  then
		begin
			if  norm_error <= (2.56e-3) * tol then 
				temp := 4.0
			else
				temp := x2y(tol / norm_error, 0.25);
			temp := temp * abs(dt);
			if  min_dt > temp  then
				temp := min_dt;
			if  dt >= 0  then
				dt := temp
			else
				dt := -temp;
		end;
	until  done;
	x1 := next_x;
end;{solve_ode}

var
	dx02: real;

begin
	x0[1] := 0.0;
	x0[2] := 1.0;
	clrscr;
	writeln;
	writeln('᮫⭠ 譮:', eps:5,
		' ⭮⥫쭠 譮:', eta:5);
	writeln;
	writeln(' ᫥:');
	writeln;
	dx02 := 0.1;
	for  k := 1  to  9  do
	begin
		writeln(' ', k);
		repeat
			write('x0 = [', x0[1]:9:9, ', ', x0[2]:9:9, ']');
			solve_ode(2, -1.0, 1.0, x0, x1);
			writeln('   x1 = [', x1[1]:9:9, ', ', x1[2]:9:9, ']');
			x0[2] := x0[2] + dx02;
		until  x1[1] >= 0.0;
		x0[2] := x0[2] - 2.0 * dx02;
		dx02 := 0.1 * dx02;
	end;
	writeln;
	write  (' <Enter>');
	readln;
end.
	